@@ -1,3 +1,4 @@
+# Copyright (c) OpenMMLab. All rights reserved.
 import torch
 import torch.nn as nn
 import torch.nn.functional as F
@@ -326,12 +327,6 @@ class SSDHead(AnchorHead):
         for i in range(num_images):
             all_anchors.append(torch.cat(anchor_list[i]))
 
-        # check NaN and Inf
-        assert torch.isfinite(all_cls_scores).all().item(), \
-            'classification scores become infinite or NaN!'
-        assert torch.isfinite(all_bbox_preds).all().item(), \
-            'bbox predications become infinite or NaN!'
-
         losses_cls, losses_bbox = multi_apply(
             self.loss_single,
             all_cls_scores,
